
# MODULE_NAME (required) and OTHER_* (optional) macros s/b defined by caller
SH_EXT		= .so
MODULE          = $(MODULE_NAME)-module$(SH_EXT)
GLUE            = $(MODULE_NAME)_glue
SO		= $(MODULE)
SO_MAJOR_NAME	= $(SO).$(SO_MAJOR_VER)
SO_MAJOR_VER	= 1
SO_MINOR_VER	= 0.0

PROFILE		= #-pg
CFLAGS		= -fPIC -g -Wall -O2 -fno-strength-reduce -fno-strict-aliasing -Wall $(OTHER_CFLAGS)
OPENMP_CFLAGS	= -fopenmp		# Used for both compile/link

CC		= /usr/bin/cc
SO_CFLAGS	= $(PROFILE) $(CFLAGS) -fPIC  -I. -I../..
SO_LINK_FLAGS	= -shared -Wl,-soname,$(SO_MAJOR_NAME)
C_COMPILE	= $(CC) $(SO_CFLAGS) $(OTHER_CFLAGS) -c
C_LINK		= $(CC) $(SO_LINK_FLAGS)
SO_DEP_LIBS	= $(DL_LIB) -lm -lc
DL_LIB		= -ldl

CXX		= /usr/bin/g++
CXXFLAGS	= $(SO_CFLAGS)
CXX_COMPILE	= $(CXX) $(CXXFLAGS) -c
CXX_LINK	= $(CXX) $(SO_LINK_FLAGS)
CXX_LIBS	= 

SLIRPROOT	= ../..
SLIRPSCRIPT	= $(SLIRPROOT)/slirp.sl
SLSH_EXE	= $(SLIRPROOT)/slsh
SLSH		= $(SLIRPROOT)/safe_slsh $(SLSH_EXE) -n
SLIRP_DEBUG	=
SLIRP		= $(SLSH) $(SLIRPSCRIPT) $(SLIRP_DEBUG)
DIFF		= diff 

SLANG_VERSION	= 20103
SLANGLIB	=   -lslang
MODULE_INSTALL_DIR = .

FC		= /usr/bin/gfortran
FCFLAGS		= -fPIC -g -O2 -fPIC
FCLIBS		= -Wl,-R/usr/lib/gcc/i686-linux-gnulp/4.4.0:/usr/lib/gcc/i686-linux-gnulp/4.4.0/../../..:/usr/lib/i686-linux-gnulp  -L/usr/lib/gcc/i686-linux-gnulp/4.4.0 -L/usr/lib/gcc/i686-linux-gnulp/4.4.0/../../.. -L/usr/lib/i686-linux-gnulp -lgfortranbegin -lgfortran -lm
FCLIBS_NAMES	=  -lgfortranbegin -lgfortran -lm
FORTRAN_COMPILE = $(FC) $(FCFLAGS) -c
FORTRAN_LINK	= $(FC)

RM		= \rm -f
CP		= \cp

all: $(PRE_ALL) $(SLSH_EXE) $(MODULE) $(OTHER_ALL)

.SUFFIXES: .cc .f

.c.o:
	$(C_COMPILE) -c $< -o $@
.cc.o:
	$(CXX_COMPILE) $< -o $@
.f.o:
	$(FORTRAN_COMPILE) $?

clean:
	$(RM) *.o *$(SH_EXT) *~ $(GLUE).c $(OTHER_CLEAN) errs.msg

distclean: clean
	$(RM) $(OTHER_DISTCLEAN)

# This is intended for internal development use only
WARN_PROTO=-Wstrict-prototypes -Wmissing-prototypes
WARN_NEX=-Wnested-externs
WARN_LLONG=-Wno-long-long
warnings:
	$(RM) $(GLUE).o
	@echo
	-CFLAGS="-Wall -W -pedantic -Winline \
		  -I. \
		$(WARN_NEX) -Wpointer-arith -Wcast-align -Wshadow \
	    $(WARN_PROTO) $(WARN_LLONG) " ; export CFLAGS ; \
	$(MAKE) -e $(GLUE).o > /dev/null 2> errs.msg
	@if [ -s errs.msg ] ; then \
	   cat errs.msg ; \
	   false ; \
	fi

$(SLSH_EXE):
	@if [ ! -x $(SLSH_EXE) ] ; then \
	   (cd $(SLIRPROOT) ; $(MAKE) slsh) ; \
	   if [ ! -x $(SLSH_EXE) ] ; then \
		echo "Cannot find/build $(SLSH_EXE)." ; \
		false; \
	   fi ; \
	fi
	@echo 
